{-----------------------------------------------------------
u_Snake.pas  v0.7
------------------------------------------------------------
Patrick G. jobe_tec@yahoo.fr
8 mai 2004
------------------------------------------------------------
Unit de gstion du serpent (Dplacement et graphisme)

Un tableau de meme longueur que le serpent contient la
position de chaque partie du corps (sprit). La case 0 du
tableau contient la position (x,y) de la tte alors que la
dernire case du tableau contient la position de la queue.

Pour faire avanc le serpent il suffit de dcaler toute les
"case" du tableau vers l'arrire, de supprimer la dernire
position, et de cre la nouvelle position de la tte 
partir de la direction du serpent.

Pour dessiner le serpent chaque case du tableau est traduite
en coordonne en pixel (en fonction de SpriteSize). Puis en
fonction de la case d'avant et d'aprs on dtermine si il
sagit d'un corps tournant, d'un corps droit, de la tte ou de
la queue et de l'orientation du sprit.

Il suffit alors de copier le sprit depuis la liste d'image
(TImageList) sur la zone de jeux (TImage.Canvas)

-----------------------------------------------------------}

unit u_Sneak;

interface

uses Types,       //dfinition du TPoint
     Graphics,    //dfinition du TBitmap
     Controls,    //gestion du Timagelist
     StdCtrls,    //gestion TLabel
     GLSound,     //Gestion des sons
     ExtCtrls;    //dfinition de Timage


type TSnakeDir =(N,E,S,O);  //dfinition du type de direction du serpent

const
  InitLength = 6;           //taille initiale du serpet
  SpriteSize = 10;          //taille des sprit 10px/10px
  ClearSprit = 14;          //index du sprite d'effacement
  TargetSprit = 15;         //index du sprit de la nourriture
  nbSprit = 16;             //nombre de sprit par serpent
  AbsScoreMax = 300;        //nb de point max par nourriture

var
  SnakeDir : TSnakeDir;     //direction du serpent
  Snake : Array of TPoint;  //contient le corps du serpent
  LastTarget : TPoint;      //position de la cible (nouriture)
  LastPos:Tpoint;           //dernire position du serpent (queue)
  DeltaTarget:integer;      //nombre de mouvement entre 2 target
  score:integer;            //score du joueur
  point:integer;            //nombre de point que peut gagner le joueur
  Gamer:word;               //defini le joueur (le dessin du serpent)
  InGame:boolean;           //Defini si une partie est en cours
  ScoreOffset:integer;      //Defini le nombre de point possible  gagner manger
  MaxPoint:word;            //nombre max de point gagner en une seule foi

Procedure InitSnake;
Procedure DrawSnake(var ACanvas:TImage;const AImg:TImageList);
procedure MoveSnake(var ACanvas:TImage;const AImg:TImageList;const Sounds:TGLSoundLibrary;const APlay:boolean);
function GameOver(const AGameZone:Timage):boolean;
procedure DrawNewTarget(var ACanvas:TImage;const AImg:TImageList);
function IsInSnake(const Apos:TPoint):boolean;
procedure SnakePause(var AEvent:TTimer;var ALabel:TLabel;var AGameZone:TImage);

implementation

//-----------------------------------------------------------
//Initialisation du serpent
//-----------------------------------------------------------
Procedure InitSnake;
var i:word;
begin
SetLength(Snake,InitLength);  //allocation memoire du tableau
for i:= 0 to InitLength-1 do  //cre le serpent en haut  droite
   begin
   Snake[i].X := InitLength-i-1;
   Snake[i].Y := 0;
   end;
SnakeDir:= S;                 //la direction par default est Sud
score :=0;                    //reset du score
DeltaTarget := 0;             //reset le nb de point a gagner
LastPos.Y := 3;               //initialisation bidon pour eviter d'effacer le serpent
end;

//-----------------------------------------------------------
//dessine le serpent
//-----------------------------------------------------------
Procedure DrawSnake(var ACanvas:TImage;const AImg:TImageList);
var i:word;
    SpritNb:integer;
begin
//efface la dernire position connue de la queue
Aimg.Draw(ACanvas.Canvas,LastPos.X*SpriteSize,LastPos.Y*SpriteSize,ClearSprit);

SpritNb := -1;
for i:=0 to length(Snake)-1 do
   begin
   if i=0 then                  //tete du serpent
     begin
     if Snake[0].Y = Snake[1].Y-1 then SpritNb := 10;
     if Snake[0].X = Snake[1].X+1 then SpritNb := 11;
     if Snake[0].Y = Snake[1].Y+1 then SpritNb := 12;
     if Snake[0].X = Snake[1].X-1 then SpritNb := 13;
     end;
   if i = length(Snake)-1 then  //queue du serpent
     begin
     if Snake[i].Y = Snake[i-1].Y-1 then SpritNb := 6;
     if Snake[i].X = Snake[i-1].X+1 then SpritNb := 7;
     if Snake[i].Y = Snake[i-1].Y+1 then SpritNb := 8;
     if Snake[i].X = Snake[i-1].X-1 then SpritNb := 9;
     end;
   if (i>0) and (i<length(Snake)-1) then //corps du serpent
     begin
     if (Snake[i].X = Snake[i-1].X-1) and (Snake[i].X = Snake[i+1].X+1) then SpritNb := 0;
     if (Snake[i].X = Snake[i-1].X+1) and (Snake[i].X = Snake[i+1].X-1) then SpritNb := 0;

     if (Snake[i].Y = Snake[i-1].Y-1) and (Snake[i].Y = Snake[i+1].Y+1) then SpritNb := 1;
     if (Snake[i].Y = Snake[i-1].Y+1) and (Snake[i].Y = Snake[i+1].Y-1) then SpritNb := 1;

     if (Snake[i].Y = Snake[i-1].Y-1) and (Snake[i].X = Snake[i+1].X-1) then SpritNb := 2;
     if (Snake[i].X = Snake[i-1].X-1) and (Snake[i].Y = Snake[i+1].Y-1) then SpritNb := 2;

     if (Snake[i].Y = Snake[i-1].Y-1) and (Snake[i].X = Snake[i+1].X+1) then SpritNb := 3;
     if (Snake[i].X = Snake[i-1].X+1) and (Snake[i].Y = Snake[i+1].Y-1) then SpritNb := 3;

     if (Snake[i].X = Snake[i-1].X+1) and (Snake[i].Y = Snake[i+1].Y+1) then SpritNb := 4;
     if (Snake[i].Y = Snake[i-1].Y+1) and (Snake[i].X = Snake[i+1].X+1) then SpritNb := 4;

     if (Snake[i].Y = Snake[i-1].Y+1) and (Snake[i].X = Snake[i+1].X-1) then SpritNb := 5;
     if (Snake[i].X = Snake[i-1].X-1) and (Snake[i].Y = Snake[i+1].Y+1) then SpritNb := 5;
     end;
   //verification d'erreur pour debugger
   //if SpritNb = -1 then ShowMessage(Erreur !);

   //efface la case
   Aimg.Draw(ACanvas.Canvas,Snake[i].X*SpriteSize,Snake[i].Y*SpriteSize,ClearSprit);
   //dessine le corps du serpent
   Aimg.Draw(ACanvas.Canvas,Snake[i].X*SpriteSize,Snake[i].Y*SpriteSize,SpritNb+(Gamer*nbSprit));
   end;
end;

//-----------------------------------------------------------
//Avance le serpent en fonction de la direction
//-----------------------------------------------------------
procedure MoveSnake(var ACanvas:TImage;const AImg:TImageList;const Sounds:TGLSoundLibrary;const APlay:boolean);
var i:word;
begin
//calcule du nombre de point
DeltaTarget := DeltaTarget+1;
if ScoreOffset >= DeltaTarget then
     point := ScoreOffset-DeltaTarget;

//test si le serpent mange la cible
if (Snake[0].X = LastTarget.X) and (Snake[0].Y = LastTarget.Y) then
  begin
  //joue le sons (plop)
  if APlay then Sounds.Samples[0].PlayOnWaveOut;
  //cre la prochaine target
  DrawNewTarget(ACanvas,AImg);
  //allonge le serpent (rallocation memoire du tableau)
  SetLength(Snake,length(Snake)+1);
  //ajoute les points au score
  if ScoreOffset >= DeltaTarget then
     Score := Score + ScoreOffset-DeltaTarget;
  //actualsie MaxPoint
  if ScoreOffset-DeltaTarget > MaxPoint then
     MaxPoint := ScoreOffset-DeltaTarget;
  //rinitialise le compteur de points
  DeltaTarget := 0;
  end;

//dplacement du serpent

//ractualise la dernire position connue
LastPos :=  Snake[length(Snake)-1];
//dcale toute les position du tableau
for i:=length(Snake)-2 downto 0 do
   Snake[i+1] := Snake[i];
//cre la nouvelle position de tte en fonction de la direction
if SnakeDir = N then
  begin
  Snake[0].X := Snake[1].X;
  Snake[0].Y := Snake[1].Y-1;
  end;
if SnakeDir = E then
  begin
  Snake[0].X := Snake[1].X+1;
  Snake[0].Y := Snake[1].Y;
  end;
if SnakeDir = S then
  begin
  Snake[0].X := Snake[1].X;
  Snake[0].Y := Snake[1].Y+1;
  end;
if SnakeDir = O then
  begin
  Snake[0].X := Snake[1].X-1;
  Snake[0].Y := Snake[1].Y;
  end;
end;

//-----------------------------------------------------------
//Verifie si il y a GameOver
//-----------------------------------------------------------
function GameOver(const AGameZone:Timage):boolean;
var i:word;
begin
//verife que le serpent ne sorte pas de la zone de jeux
result :=  ((Snake[0].X < 0) or (Snake[0].X >= AGameZone.Width/SpriteSize) or
            (Snake[0].Y < 0) or (Snake[0].Y >= AGameZone.Height/SpriteSize));

//verifie que le serpent ne se mange pas lui mme
for i:=1 to length(Snake)-1 do
  begin
  result := result or ((Snake[0].X = Snake[i].X) and (Snake[0].Y = Snake[i].Y));
  end
end;


//-----------------------------------------------------------
//Cre une nouvelle cible (nourriture)
//-----------------------------------------------------------
procedure DrawNewTarget(var ACanvas:TImage;const AImg:TImageList);
begin
//efface la dernire target
Aimg.Draw(ACanvas.Canvas,LastTarget.X*SpriteSize,LastTarget.Y*SpriteSize,ClearSprit);
//cre une nouvelle target tant qu'elle n'est pas sous le serpent
repeat
  LastTarget.X := random(ACanvas.Width div SpriteSize);
  LastTarget.Y := random(ACanvas.Height div SpriteSize);
until not IsInSnake(LastTarget);
//dessine la nouvelle target
Aimg.Draw(ACanvas.Canvas,LastTarget.X*SpriteSize,LastTarget.Y*SpriteSize,TargetSprit+(Gamer*nbSprit));
end;

//-----------------------------------------------------------
//verifie si les coordonne transmises son dans le corps du serpent
//-----------------------------------------------------------
function IsInSnake(const Apos:TPoint):boolean;
var i:word;
begin
result := false;
for i:=0 to length(Snake)-1 do
  result := result or ((APos.X = Snake[i].X) and (APos.Y = Snake[i].Y));
result := result or ((APos.X = LastPos.X) and (APos.Y = LastPos.Y));
end;

//-----------------------------------------------------------
//Gestion du mode pause
//-----------------------------------------------------------
procedure SnakePause(var AEvent:TTimer;var ALabel:TLabel;var AGameZone:TImage);
begin
if InGame then
  begin
  //dactive le timer de jeux
  AEvent.Enabled := not AEvent.Enabled;
  //affiche le label pause au centre de l'crant
  ALabel.Top := (AGAmeZone.Height-ALabel.Height) div 2;
  ALabel.Left := (AGAmeZone.Width-ALabel.Width) div 2;
  ALabel.Visible := not AEvent.Enabled;
  end;
end;

end.    
